"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Preview = void 0;
const command_1 = require("@salesforce/command");
const core_1 = require("@salesforce/core");
const AndroidEnvironmentRequirements_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/AndroidEnvironmentRequirements");
const AndroidLauncher_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/AndroidLauncher");
const Common_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/Common");
const CommonUtils_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/CommonUtils");
const IOSEnvironmentRequirements_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/IOSEnvironmentRequirements");
const IOSLauncher_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/IOSLauncher");
const PreviewUtils_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/PreviewUtils");
const Requirements_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/Requirements");
const LwrServerUtils_1 = require("../../../../../common/LwrServerUtils");
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const configSchema = __importStar(require("./previewConfigurationSchema.json"));
core_1.Messages.importMessagesDirectory(__dirname);
const messages = core_1.Messages.loadMessages('@salesforce/lwc-dev-mobile', 'preview');
class Preview extends command_1.SfdxCommand {
    constructor() {
        super(...arguments);
        this.deviceName = '';
        this.componentName = '';
        this.targetApp = '';
        this.projectDir = '';
        this.configFilePath = '';
        this._requirements = {};
    }
    async run() {
        this.logger.info(`Preview command invoked for ${this.flags.platform}`);
        return this.validateInputParameters()
            .then(() => {
            return Requirements_1.RequirementProcessor.execute(this.commandRequirements);
        })
            .then(() => {
            this.logger.info('Setup requirements met, continuing with preview');
            return this.launchPreview();
        })
            .catch((error) => {
            this.logger.warn(`Preview failed for ${this.flags.platform}.`);
            return Promise.reject(error);
        });
    }
    async validateInputParameters() {
        this.deviceName = this.flags.target.trim();
        this.componentName = this.flags.componentname.trim();
        this.targetApp = this.flags.targetapp.trim();
        this.projectDir = CommonUtils_1.CommonUtils.resolveUserHomePath(this.flags.projectdir.trim());
        const configFileName = CommonUtils_1.CommonUtils.resolveUserHomePath(this.flags.configfile.trim());
        this.configFilePath = path_1.default.normalize(path_1.default.resolve(this.projectDir, configFileName));
        const hasConfigFile = configFileName.length > 0 && fs_1.default.existsSync(this.configFilePath);
        const isBrowserTargetApp = PreviewUtils_1.PreviewUtils.isTargetingBrowser(this.targetApp);
        this.logger.debug('Validating Preview command inputs.');
        const platform = this.flags.platform.trim();
        if (!Common_1.CommandLineUtils.platformFlagIsValid(platform, true)) {
            return Promise.reject(new core_1.SfdxError(messages.getMessage('error:invalidInputFlagsDescription'), 'lwc-dev-mobile', Preview.examples));
        }
        if (Common_1.CommandLineUtils.platformFlagIsDesktop(platform) &&
            !isBrowserTargetApp) {
            return Promise.reject(new core_1.SfdxError(messages.getMessage('error:invalidTargetAppForDesktop'), 'lwc-dev-mobile', Preview.examples));
        }
        if (isBrowserTargetApp && hasConfigFile) {
            this.logger.warn(messages.getMessage('ignoringConfigFileFlagDescription'));
        }
        if (this.componentName.length === 0) {
            return Promise.reject(new core_1.SfdxError(messages.getMessage('error:invalidComponentNameFlagsDescription'), 'lwc-dev-mobile', Preview.examples));
        }
        if (isBrowserTargetApp === false && hasConfigFile === false) {
            return Promise.reject(new core_1.SfdxError(messages.getMessage('error:invalidConfigFile:missingDescription', [this.configFilePath]), 'lwc-dev-mobile', Preview.examples));
        }
        if (isBrowserTargetApp === false && hasConfigFile === true) {
            const validationResult = await PreviewUtils_1.PreviewUtils.validateConfigFileWithSchema(this.configFilePath, configSchema);
            if (validationResult.passed === false) {
                return Promise.reject(new core_1.SfdxError(messages.getMessage('error:invalidConfigFile:genericDescription', [this.configFilePath, validationResult.errorMessage]), 'lwc-dev-mobile', Preview.examples));
            }
            const configFileContent = PreviewUtils_1.PreviewUtils.loadConfigFile(this.configFilePath);
            this.appConfig = configFileContent.getAppConfig(this.flags.platform, this.targetApp);
            if (this.appConfig === undefined) {
                const errMsg = messages.getMessage('error:invalidConfigFile:missingAppConfigDescription', [this.targetApp, this.flags.platform]);
                return Promise.reject(new core_1.SfdxError(messages.getMessage('error:invalidConfigFile:genericDescription', [this.configFilePath, errMsg]), 'lwc-dev-mobile', Preview.examples));
            }
        }
        return Promise.resolve();
    }
    async init() {
        if (this.logger) {
            return Promise.resolve();
        }
        Common_1.CommandLineUtils.flagFailureActionMessages = Preview.examples;
        return super
            .init()
            .then(() => core_1.Logger.child('force:lightning:lwc:preview', {}))
            .then((logger) => {
            this.logger = logger;
            return Promise.resolve();
        });
    }
    _help() {
        const isCommandHelp = this.argv.filter((v) => v.toLowerCase() === '-h' || v.toLowerCase() === '--help').length > 0;
        if (isCommandHelp) {
            super._help();
        }
        else {
            const message = messages.getMessage('configFileHelpDescription');
            console.log(`${message}`);
        }
        return this.exit(0);
    }
    get commandRequirements() {
        if (Object.keys(this._requirements).length === 0) {
            const requirements = {};
            if (!Common_1.CommandLineUtils.platformFlagIsDesktop(this.flags.platform)) {
                requirements.setup = Common_1.CommandLineUtils.platformFlagIsAndroid(this.flags.platform)
                    ? new AndroidEnvironmentRequirements_1.AndroidEnvironmentRequirements(this.logger, this.flags.apilevel)
                    : new IOSEnvironmentRequirements_1.IOSEnvironmentRequirements(this.logger);
                this._requirements = requirements;
            }
        }
        return this._requirements;
    }
    async launchPreview() {
        let appBundlePath;
        if (PreviewUtils_1.PreviewUtils.isTargetingBrowser(this.targetApp) === false &&
            this.appConfig) {
            try {
                CommonUtils_1.CommonUtils.startCliAction(messages.getMessage('previewAction'), messages.getMessage('previewFetchAppBundleStatus'));
                appBundlePath = PreviewUtils_1.PreviewUtils.getAppBundlePath(path_1.default.dirname(this.configFilePath), this.appConfig);
            }
            catch (error) {
                CommonUtils_1.CommonUtils.stopCliAction(messages.getMessage('previewFetchAppBundleFailureStatus'));
                return Promise.reject(error);
            }
        }
        if (Common_1.CommandLineUtils.platformFlagIsDesktop(this.flags.platform)) {
            return this.launchDesktop(this.componentName, this.projectDir);
        }
        else if (Common_1.CommandLineUtils.platformFlagIsIOS(this.flags.platform)) {
            const config = this.appConfig && this.appConfig;
            return this.launchIOS(this.deviceName, this.componentName, this.projectDir, appBundlePath, this.targetApp, config);
        }
        else {
            const config = this.appConfig && this.appConfig;
            return this.launchAndroid(this.deviceName, this.componentName, this.projectDir, appBundlePath, this.targetApp, config);
        }
    }
    async launchDesktop(componentName, projectDir) {
        return LwrServerUtils_1.LwrServerUtils.startLwrServer(componentName, projectDir).then((serverPort) => CommonUtils_1.CommonUtils.launchUrlInDesktopBrowser(`http://localhost:${serverPort}`));
    }
    async launchIOS(deviceName, componentName, projectDir, appBundlePath, targetApp, appConfig) {
        const launcher = new IOSLauncher_1.IOSLauncher(deviceName);
        return LwrServerUtils_1.LwrServerUtils.startLwrServer(componentName, projectDir).then((serverPort) => launcher.launchPreview(componentName, projectDir, appBundlePath, targetApp, appConfig, serverPort));
    }
    async launchAndroid(deviceName, componentName, projectDir, appBundlePath, targetApp, appConfig) {
        const launcher = new AndroidLauncher_1.AndroidLauncher(deviceName);
        return LwrServerUtils_1.LwrServerUtils.startLwrServer(componentName, projectDir).then((serverPort) => launcher.launchPreview(componentName, projectDir, appBundlePath, targetApp, appConfig, serverPort));
    }
}
exports.Preview = Preview;
Preview.description = messages.getMessage('commandDescription');
Preview.args = [{ name: 'file' }];
Preview.examples = [
    `$ sfdx force:lightning:lwc:preview -p Desktop -n HelloWordComponent`,
    `$ sfdx force:lightning:lwc:preview -p iOS -t LWCSim2 -n HelloWordComponent`,
    `$ sfdx force:lightning:lwc:preview -p Android -t LWCEmu2 -n HelloWordComponent`
];
Preview.flagsConfig = {
    platform: command_1.flags.string({
        char: 'p',
        description: `Specify platform ('Desktop' or 'iOS' or 'Android').`,
        required: true
    }),
    componentname: command_1.flags.string({
        char: 'n',
        description: messages.getMessage('componentnameFlagDescription'),
        required: true
    }),
    configfile: command_1.flags.string({
        char: 'f',
        description: messages.getMessage('configFileFlagDescription'),
        required: false,
        default: ''
    }),
    confighelp: command_1.flags.help({
        default: false,
        description: messages.getMessage('configHelpFlagDescription'),
        required: false
    }),
    projectdir: command_1.flags.string({
        char: 'd',
        description: messages.getMessage('projectDirFlagDescription'),
        required: false,
        default: process.cwd()
    }),
    target: command_1.flags.string({
        char: 't',
        description: messages.getMessage('targetFlagDescription'),
        required: false,
        default: 'SFDXDebug'
    }),
    targetapp: command_1.flags.string({
        char: 'a',
        description: messages.getMessage('targetAppFlagDescription'),
        required: false,
        default: PreviewUtils_1.PreviewUtils.BROWSER_TARGET_APP
    })
};
Preview.requiresUsername = false;
Preview.supportsDevhubUsername = false;
Preview.requiresProject = false;
//# sourceMappingURL=preview.js.map